@@ -8,9 +8,12 @@ use Carp         ();
 use Data::Dumper ();
 use List::Util   ();
 use Scalar::Util ();
+use Sub::Util    ();
 
 use experimental 'signatures';
 
+our $VERSION = '0.006';
+
 # The main entity registry, inspired by https://github.com/skypjack/entt
 
 use constant {
@@ -29,10 +32,35 @@ use constant {
 
 ## Entity "methods"
 
-my $version = sub ($e) { $e >> ENTITY_SHIFT };
-my $entity  = sub ($e) { $e &  ENTITY_MASK  };
-my $is_null = sub ($e) { $e->$entity == NULL_ENTITY };
-my $format  = sub ($e) { sprintf '%012b:%020b', $e->$version, $e->$entity };
+my $base = __PACKAGE__ . '::';
+my $version = Sub::Util::set_subname "${base}GUID::version" => sub ($e) { $e >> ENTITY_SHIFT };
+my $entity  = Sub::Util::set_subname "${base}GUID::entity"  => sub ($e) { $e &  ENTITY_MASK  };
+my $is_null = Sub::Util::set_subname "${base}GUID::is_null" => sub ($e) { $e->$entity == NULL_ENTITY };
+my $format  = Sub::Util::set_subname "${base}GUID::format"  => sub ($e) { sprintf '%03X:%05X', $e->$version, $e->$entity };
+
+## Sparse set "methods"
+
+my $swap_components = Sub::Util::set_subname "${base}Set::swap_components" => sub ( $set, $le, $re ) {
+    my ( $ld, $rd ) = @{ $set->[SPARSE] }[ $le, $re ];
+    @{ $set->[COMPONENTS] }[ $ld, $rd ] = @{ $set->[COMPONENTS] }[ $rd, $ld ];
+};
+
+my $swap = Sub::Util::set_subname "${base}Set::swap" => sub ( $set, $le, $re ) {
+    my ( $ld, $rd ) = @{ $set->[SPARSE] }[ $le, $re ];
+    my ( $ls, $rs ) = @{ $set->[DENSE ] }[ $ld, $rd ];
+
+    Carp::confess "Cannot swap $le and $re: they are not members of the set"
+        unless $ls == $le && $rs == $re;
+
+    $set->$swap_components( $le, $re );
+    @{ $set->[DENSE ] }[ $ld, $rd ] = @{ $set->[DENSE ] }[ $rd, $ld ];
+    @{ $set->[SPARSE] }[ $ls, $rs ] = @{ $set->[SPARSE] }[ $rs, $ls ];
+};
+
+my $contains = Sub::Util::set_subname "${base}Set::contains" => sub ( $set, $index ) {
+    my $sparse = $set->[SPARSE][$index] // return;
+    return ( $set->[DENSE][$sparse] // $index + 1 ) == $index;
+};
 
 ## Private, hidden methods
 
@@ -139,8 +167,8 @@ sub check ( $self, $guid, $type ) {
 
     my $index  = $guid->$entity;
     my $set    = $self->{components}{"$type"};
-    my $sparse = $set->[SPARSE][$index] // return;
-    return ( $set->[DENSE][$sparse] // $index + 1 ) == $index;
+
+    $set->$contains( $index );
 }
 
 # Add or replace a component for an entity
@@ -251,6 +279,63 @@ sub valid ( $self, $guid ) {
         && ( $self->{entities}[$pos] // $guid + 1 ) == $guid;
 }
 
+sub sort ( $self, $name, $comparator ) {
+    my $set = $self->{components}{$name}
+        // Carp::croak "Cannot sort $name: no such component in registry";
+
+    my $sparse = $set->[SPARSE];
+    my $dense  = $set->[DENSE];
+    my $comps  = $set->[COMPONENTS];
+
+    # Sorting a component invalidates any cached view that uses it
+    delete $self->{view_cache}{$_} for
+        grep { index( $_, "|$name|" ) != -1 }
+        keys %{ $self->{view_cache} };
+
+    if ( ! ref $comparator ) {
+        my $other = $self->{components}{$comparator}
+            // Carp::croak "Cannot sort according to $comparator: no such component in registry";
+
+        my $j = 0;
+        for my $i ( 0 .. $#{ $other->[DENSE] } ) {
+            last if $i > $#{ $dense };
+
+            my $this = $dense->[$j]        // die "Undefined in set";
+            my $that = $other->[DENSE][$i] // die 'Undefined in other';
+
+            next unless $set->$contains($that);
+
+            $set->$swap( $this, $that ) unless $this == $that;
+            $j++;
+        }
+
+        return;
+    }
+
+    # See https://skypjack.github.io/2019-09-25-ecs-baf-part-5/
+    my $caller = caller;
+    @$dense = sort {
+        no strict 'refs';
+        local ${ $caller . '::a' } = $comps->[ $sparse->[ $a ] ];
+        local ${ $caller . '::b' } = $comps->[ $sparse->[ $b ] ];
+        $comparator->()
+    } @$dense;
+
+    for my $curr ( 0 .. $#$dense ) {
+        my $next = $sparse->[ $dense->[ $curr ] ];
+
+        while ( $next != $curr ) {
+            $set->$swap_components( @{ $dense }[ $curr, $next ] );
+
+            $sparse->[ $dense->[ $curr ] ] = $curr;
+            $curr = $next;
+            $next = $sparse->[ $dense->[ $curr ] ];
+        }
+    }
+
+    return;
+}
+
 package
     Game::Entities::View {
     no overloading;
@@ -333,7 +418,8 @@ sub _dump_entities ( $self, @types ) {
     local $Data::Dumper::Terse  = 1;
     local $Data::Dumper::Indent = 0;
 
-    my @names = @types || keys %{ $self->{components} };
+    my @names = @types;
+    @names = sort keys %{ $self->{components} } unless @types;
 
     my $print = ! defined wantarray;
     open my $fh, '>', \my $out or $print = 1;
